﻿using System; // Activator
using System.Drawing; // Color
using System.Windows.Forms; // Button
using System.ComponentModel; // TypeDescriptor
using System.Reflection;  // MethodInfo

namespace Builder
{
    public class Factory : Button
    {
        public string componentName { get; set; }
        public Color componentColor { get; set; }
        public Type componentType { get; set; }

        public Factory ()
        {
            this.Size = new Size (110, 46);
            this.MouseDown += this.Factory_MouseDown;

        }

        public Factory (Type type0, string name0 = "", Color color0 = default (Color)) : this ()
        {
            componentType = type0;
            componentName = name0;
            componentColor = color0;

            if (componentName == "")
                componentName = componentType.Name;
            this.Text = componentName;

            ToolTip tooltip = new ToolTip ();
            tooltip.SetToolTip (this, componentName);

            AttributeCollection attrCol = TypeDescriptor.GetAttributes (componentType);
            ToolboxBitmapAttribute imageAttr = attrCol [typeof (ToolboxBitmapAttribute)] as ToolboxBitmapAttribute;
            if (imageAttr != null)
            {
                this.Image = imageAttr.GetImage (componentType);
                this.Text = "";
                // this.TextAlign = ContentAlignment.MiddleRight;
            }

            MethodInfo meth = componentType.GetMethod ("componentColor", BindingFlags.Public | BindingFlags.Static);
            if (meth != null)
            {
                Object result = meth.Invoke (null, null);
                this.BackColor = (Color)result;
            }
        }

        private void Factory_MouseDown (object sender, MouseEventArgs e)
        {
            DoDragDrop (this, DragDropEffects.Copy);
        }

        public virtual object createInstance ()
        {
            return Activator.CreateInstance (componentType);
        }
    }
}
